@@ -46,13 +46,17 @@ struct _UmPhotoDialog {
         GtkWidget *popup_button;
         GtkWidget *take_picture_button;
         GtkWidget *flowbox;
+        GtkWidget *recent_pictures;
 
 #ifdef HAVE_CHEESE
         CheeseCameraDeviceMonitor *monitor;
         guint num_cameras;
 #endif /* HAVE_CHEESE */
 
+        GListStore *recent_faces;
         GListStore *faces;
+        GFile *generated_avatar;
+        gboolean custom_avatar_was_chosen;
 
         SelectAvatarCallback *callback;
         gpointer              data;
@@ -80,6 +84,7 @@ webcam_response_cb (GtkDialog     *dialog,
                 pb2 = gdk_pixbuf_scale_simple (pb, 96, 96, GDK_INTERP_BILINEAR);
 
                 um->callback (pb2, NULL, um->data);
+                um->custom_avatar_was_chosen = TRUE;
 
                 g_object_unref (pb2);
                 g_object_unref (pb);
@@ -143,21 +148,37 @@ face_widget_activated (GtkFlowBox      *flowbox,
         filename = g_object_get_data (G_OBJECT (image), "filename");
 
         um->callback (NULL, filename, um->data);
+        um->custom_avatar_was_chosen = TRUE;
 
         gtk_popover_popdown (GTK_POPOVER (um));
 }
 
+static void
+generated_avatar_activated (GtkFlowBox      *flowbox,
+                            GtkFlowBoxChild *child,
+                            UmPhotoDialog   *um)
+{
+        face_widget_activated (flowbox, child, um);
+        um->custom_avatar_was_chosen = FALSE;
+}
+
 static GtkWidget *
 create_face_widget (gpointer item,
                     gpointer user_data)
 {
+        GdkPixbuf *pixbuf = NULL;
         GtkWidget *image;
-        GIcon *icon;
 
-        icon = g_file_icon_new (G_FILE (item));
-        image = gtk_image_new_from_gicon (icon, GTK_ICON_SIZE_DIALOG);
+        pixbuf = gdk_pixbuf_new_from_file_at_size (g_file_get_path (G_FILE (item)),
+                                                   AVATAR_PIXEL_SIZE,
+                                                   AVATAR_PIXEL_SIZE,
+                                                   NULL);
+        if (pixbuf == NULL)
+                return NULL;
+
+        image = gtk_image_new_from_pixbuf (round_image (pixbuf));
+        g_object_unref (pixbuf);
         gtk_image_set_pixel_size (GTK_IMAGE (image), AVATAR_PIXEL_SIZE);
-        g_object_unref (icon);
 
         gtk_widget_show (image);
 
@@ -187,6 +208,16 @@ setup_photo_popup (UmPhotoDialog *um)
         g_signal_connect (um->flowbox, "child-activated",
                           G_CALLBACK (face_widget_activated), um);
 
+        um->recent_faces = g_list_store_new (G_TYPE_FILE);
+        gtk_flow_box_bind_model (GTK_FLOW_BOX (um->recent_pictures),
+                                 G_LIST_MODEL (um->recent_faces),
+                                 create_face_widget,
+                                 um,
+                                 NULL);
+        g_signal_connect (um->recent_pictures, "child-activated",
+                          G_CALLBACK (generated_avatar_activated), um);
+        um->custom_avatar_was_chosen = FALSE;
+
         dirs = g_get_system_data_dirs ();
         for (i = 0; dirs[i] != NULL; i++) {
                 g_autoptr(GFileEnumerator) enumerator = NULL;
@@ -267,6 +298,34 @@ on_popup_button_button_pressed (GtkToggleButton *button,
         return FALSE;
 }
 
+void
+um_photo_dialog_generate_avatar (UmPhotoDialog *um,
+                                 const gchar   *name)
+{
+        cairo_surface_t *surface;
+        gchar *filename;
+
+        surface = generate_user_picture (name);
+
+        /* Save into a tmp file that later gets copied by AccountsService */
+        filename = g_build_filename (g_get_user_runtime_dir (), "avatar.png", NULL);
+        um->generated_avatar = g_file_new_for_path (filename);
+        cairo_surface_write_to_png (surface, g_file_get_path (um->generated_avatar));
+        g_free (filename);
+
+        /* Overwrite the first item */
+        if (g_list_model_get_item (G_LIST_MODEL (um->recent_faces), 0) != NULL)
+                g_list_store_remove (um->recent_faces, 0);
+
+        g_list_store_insert (um->recent_faces, 0,
+                             um->generated_avatar);
+        gtk_widget_show_all (um->recent_pictures);
+
+        if (!um->custom_avatar_was_chosen) {
+                um->callback (NULL, g_file_get_path (um->generated_avatar), um->data);
+        }
+}
+
 UmPhotoDialog *
 um_photo_dialog_new (GtkWidget            *button,
                      SelectAvatarCallback  callback,
@@ -319,6 +378,7 @@ um_photo_dialog_class_init (UmPhotoDialogClass *klass)
         gtk_widget_class_set_template_from_resource (wclass, "/org/gnome/initial-setup/gis-account-avatar-chooser.ui");
 
         gtk_widget_class_bind_template_child (wclass, UmPhotoDialog, flowbox);
+        gtk_widget_class_bind_template_child (wclass, UmPhotoDialog, recent_pictures);
         gtk_widget_class_bind_template_child (wclass, UmPhotoDialog, take_picture_button);
 #ifdef HAVE_CHEESE
         gtk_widget_class_bind_template_callback (wclass, webcam_icon_selected);
